/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.Button");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");

(function() {

var preloaded = false;

/*
 * Constructor:
 * var button = new net.jazz.ajax.ui.internal.Button({args});
 * 
 * args:
 * 
 *  href: string (href will supercede callback if both passed in)
 * 		OR
 * 	callback: string OR function
 * 	
 * 	buttonText: string
 * 	imageClass: string
 * 	size: "small" or "large"
 *  dropDownValues: Array of objects with: { href or callback, id, label, iconUrl }. default null (no drop down).
 * 	boldFirstWord: boolean
 * 	disabled: boolean
 * 
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.Button",
	[dijit._Widget, dijit._Templated], {

	/* Defaults */
	
	buttonText: "",
	imageClass: "",
	size: "small",
	dropDownValues: null,
	boldFirstWord: false,
	disabled: false,

	templatePath: dojo.moduleUrl("jazz.ui", "templates/Button.html"),
	
	constructor: function() {
		if (!preloaded) {
			setTimeout(_preloadImages,2000);
			preloaded = true;
		}
	},

	postCreate: function() {
		if (this.size === "large") {
			dojo.addClass(this._buttonWrapper,"largeButton");
		} else {
			dojo.addClass(this._buttonWrapper,"smallButton");
		}
		
		dojo.addClass(this._image,this.imageClass);
		this._updateDisabled();
		
		if (this.boldFirstWord) {
			var firstSpace = this.buttonText.indexOf(" ");
			this._label.innerHTML = "<b>" + this.buttonText.substring(0,firstSpace) + "</b>" + this.buttonText.substring(firstSpace);
		} else {
			this._label.innerHTML = this.buttonText;
		}
		if (this.href) this.domNode.href = this.href;
		if (this.dropDownValues) this.populateDropDown(this.dropDownValues);
	},
	populateDropDown: function(values) {
		if (values) {
			this._menuArrow.style.display = "block";	
		} else {
			this._menuArrow.style.display = "none";
		}
		this.dropDownValues = values;
		this._menuArrow.tabIndex = 0;
		this.connect(this._menuArrow, "onkeypress", this._handleArrowKeyPress);
		dojo.addClass(this.domNode,"jazz-ui-Button-hasMenu");
	},
	_handleArrowKeyPress: function(e) {
		var characterCode = e.keyCode;
		if(characterCode == dojo.keys.ENTER || characterCode == dojo.keys.DOWN_ARROW || characterCode == dojo.keys.SPACE) {
			this._openMenu(e);
		}	
	},
	setDisabled: function (disabled) {
		this.disabled = disabled;
		this._updateDisabled();
	},
	destroy: function() {
		this.inherited(arguments);
	},
	_updateDisabled: function() {
		if (this.disabled) {
			dojo.addClass(this.domNode,"jazz-ui-Button-disabled");
		} else {
			dojo.removeClass(this.domNode,"jazz-ui-Button-disabled");
		}
	},
	makeUrlsUnique: function() {
		var ts = Date.parse(new  Date().toString());
		if(this.href && this._button) {
			this.href = this.href.replace(/&ts=([0-9]+)/, "") + "&ts=" + ts + "0";
			this._button.href = this.href;
		}
		if(this.dropDownValues) {
			for(var i=0;i< this.dropDownValues.length;i++) {
				if (this.dropDownValues[i]) {
					this.dropDownValues[i].url = this.dropDownValues[i].url.replace(/&ts=([0-9]+)/, "") + "&ts=" + ts + "" + (i+1);
					if(this.dropDownValues[i].menuItem) {
						this.dropDownValues[i].menuItem.url =this.dropDownValues[i].url 
					}
				}
			}
		}
	}, 
	_handleClick: function(event) {
		if (this.disabled) {
			event.preventDefault();
			this.domNode.blur();
			return;
		}
		if (!this.href) {
			event.preventDefault();
			this.callback(event);	
		} else if (dojo.isIE) {
			event.preventDefault();
			location.href = this.href;
		}
		// else if firefox && href, let the link fire
	},
	_openMenu: function (event) {
		if (this.menuOpen) return;
		this.menuOpen = true;
		dojo.stopEvent(event);
		this._button.blur();
		if (!this._buttonMenu) {
			var menu = new dijit.Menu({onExecute: dojo.hitch(this,this._closeMenu)});
			dojo.addClass(menu.domNode,"jazz-ui-Button-Menu");
			this._menu = menu;
			var dropDownValues = this.dropDownValues;
			for (var i = 0; i < dropDownValues.length; i++) {
				if (dropDownValues[i].callback) {
					var id = dropDownValues[i].id;
					var item = new dijit.MenuItem({
						label: dropDownValues[i].label,
						onClick: function(event){
							this.dropDownValue.callback(event,this.dropDownValue);
						},
						dropDownValue: dropDownValues[i]
					})
				} else {
					var item = new dijit.MenuItem({label: dropDownValues[i].label, onClick: function() {location.href = this.url}})
					item.url = this.dropDownValues[i].url;	
				}
				menu.addChild(item);
				if (dropDownValues[i].iconUrl) {
					var img = new Image();
					img.src = dropDownValues[i].iconUrl;
					item.iconNode.appendChild(img);	
				}
				this.dropDownValues[i].menuItem = item;
			}
			var buttonMenu = new jazz.ui.ButtonMenu();
			buttonMenu._menuArea.appendChild(this._menu.domNode);
			this._buttonMenu = buttonMenu;
		} 
		document.body.appendChild(this._buttonMenu.domNode);
		var pos = dojo.coords(this._buttonWrapper, true);
		this._menu.domNode.style.width = pos.w - 4 + "px";
		this._buttonMenu.domNode.style.paddingTop = pos.h + "px";
		this._buttonMenu.domNode.style.top = pos.y - 3 + "px";
		this._buttonMenu.domNode.style.left = pos.x + "px";
		if (!this._menuCloseOnBody)
			this._menuCloseOnBody = this.connect(document.body,"onclick",this._closeMenu);
		if (!this._menuCloseOnMenu)
			this._menuCloseOnMenu = this.connect(this,"_onblur",this._closeMenu);
		if (!this._menuCloseOnEsc)
			this._menuCloseOnEsc = this.connect(document.body,"onkeypress",this._handleKeypress);
		this._menu.startup();
		this._menu.focus();
	},
	_handleKeypress: function(e) {
		var k = dojo.keys;
		switch (e.charOrCode) {
			case k.ESCAPE:
				this._closeMenu(e);
				break;
		}
	},
	_closeMenu: function(event) {
		document.body.removeChild(this._buttonMenu.domNode);
		this.disconnect(this._menuCloseOnBody);
		this.disconnect(this._menuCloseOnMenu);
		this.disconnect(this._menuCloseOnEsc);
		this._menuCloseOnBody = null;
		this._menuCloseOnMenu = null;
		this._menuCloseOnEsc = null;
		this.menuOpen = false;
		this._menuArrow.focus();
	}
});

dojo.declare("jazz.ui.ButtonMenu",
	[dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("jazz.ui", "templates/ButtonMenu.html")
});

var pictures = new Array();
function _preloadImages () {
	var versionId = (dojo.dependency !== undefined) ? "/_ver"+dojo.dependency.getWebBundleVersion("net.jazz.web.ui") : "/_ver";
	pictures.push((new Image()).src = dojo.moduleUrl("jazz.ui", "graphics/Button/button-bg-hov.png"+versionId));
}
})();