/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.ItemList");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");

function removeChildren(e){
	while(e.hasChildNodes()){
		e.removeChild(e.firstChild);
	}
}

function isNode(o){
	return o.nodeType && o.cloneNode;
}

function getRow(evt){
	var e = evt.target;
	while(e.parentNode){
		if(e.tagName === "TR"){
			return e;
		}
		e = e.parentNode;
	}
	return null;
}

var _statusMap = {
	errorMsg: "ERROR",
	warningMsg: "WARNING",
	pendingMsg: "INFO",
	successMsg: "OK"
}

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.ItemList", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/templates/ItemList.html"),
	
	addNewButton: function(actionHref, label){
		var button = document.createElement("button");
		button.appendChild(document.createTextNode(label));
		button.__jazzActionHref = "#action=" + actionHref;
		this.newItemButtonArea.appendChild(button);
		return dojo.connect(button,"onclick",this,"_handleNewItemButtonClick");
	},
	
	setHeaderRow: function(/*Array*/labels){
		removeChildren(this.headerRowLine);
		removeChildren(this.headerRow);
		var self = this;
		dojo.forEach(labels, function(label){
			var th = document.createElement("th");
			th.appendChild(document.createTextNode(label));
			self.headerRow.appendChild(th);
			th = document.createElement("th");
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" "));
			}
			self.headerRowLine.appendChild(th);
		});
	},
	
	addRow: function(actionHref, /*Array*/columns){
		var row = document.createElement("tr");
		if (actionHref != null) {
			row.__jazzActionHref = "#action=" + actionHref;
		}
		this.tbody.appendChild(row);
		for(var i=0; i<columns.length; i++){
			var td = row.insertCell(i);
			if(i===0 && row.__jazzActionHref != null){
				var link = document.createElement("a");
				link.href = row.__jazzActionHref;
				dojo.connect(link, "onfocus", this._handleFocus);
				dojo.connect(link, "onblur", this._handleMouseOut);
				td.appendChild(link);
				td = link;
			}
			var o = columns[i];
			if(dojo.isString(o)){
				td.appendChild(document.createTextNode(o));
			} else if(isNode(o)){
				td.appendChild(o);
			}
		}
		return row;
	},
	
	handleError: function(errorObj){
		if (this._usingWizardHeader()) {
			this.header.displayServiceCallbackErrorMessage(errorObj);
		} else {
			this.displayErrorMessage(errorObj.message,false);
		}
	},
	
	_handleNewItemButtonClick: function(e){
		if(e.target.__jazzActionHref){
			window.location.href = e.target.__jazzActionHref;
		}
	},
	
	_handleRowClick: function(e){
		var row = getRow(e);
		if (row.__jazzActionHref != null) {
			if (e.target.tagName === "A") {
				window.location.href = row.__jazzActionHref;
			}
		}
	},
	
	_handleFocus: function(e) {
		var row = getRow(e);
		if (row && (!row.collapsable || e.target.tagName === "A")) {
			dojo.toggleClass(row, "selected", true);
		}
	},
	
	_handleMouseOver: function(e) {
		//ensure that no other rows are "selected" because of keyboard focus
		var rows = this.tbody.rows;
		var i;
		for (i=0; i<rows.length; i++) {
			dojo.toggleClass(rows.item(i), "selected", false);
		}
		
		//highlight the current row
		var row = getRow(e);
		if (row && (!row.collapsable || e.target.tagName === "A")) {
			dojo.toggleClass(row, "selected", true);
		}
	},
	
	_handleMouseOut: function(e){
		var row = getRow(e);
		if (row && (!row.collapsable || e.target.tagName === "A")) {
			dojo.toggleClass(row, "selected", false);
		}
	},
	
	displaySuccessMessage: function (msg, isDomNode) {
		this.displayMessage(msg,"successMsg", isDomNode);
	},
	
	displayPendingMessage: function (msg, isDomNode) {
		this.displayMessage(msg,"pendingMsg", isDomNode);
	},
	
	displayWarningMessage: function (msg, isDomNode) {
		this.displayMessage(msg,"warningMsg", isDomNode);
	},
	
	displayErrorMessage: function (msg, isDomNode) {
		this.displayMessage(msg, "errorMsg", isDomNode);
	},
	
	displayMessage: function (msg, cssClass, isDomNode) {
		if (this._usingWizardHeader()) {
			var status = {
				type: _statusMap[cssClass],
				summary: msg,
				isDomNode: isDomNode 
			};
			this.header.showStatus(status);
		}
		else {
			dojox.data.dom.removeChildren(this.messageBox);
			this.messageBox.className = cssClass;
			if (isDomNode) {
				this.messageBox.appendChild(msg);
			}
			else {
				this.messageBox.appendChild(document.createTextNode(msg));	
			}
		}
	},
	
	clearMessage: function() {
		if (this._usingWizardHeader()) {
			this.header.clearMessages();
		}
		else {
			dojox.data.dom.removeChildren(this.messageBox);
			this.messageBox.className = "";
		}
	},

	_usingWizardHeader: function() {
		if (this.header)
			return true;
		else
			return false;
	}
});

})();